@@ -1,12 +1,12 @@
 // Take a look at the license at the top of the repository in the LICENSE file.
 
 #[cfg(unix)]
-use std::os::unix::io::{AsRawFd, FromRawFd, RawFd};
+use std::os::unix::io::{AsFd, AsRawFd, BorrowedFd, FromRawFd, RawFd};
 
 use crate::ffi;
 use glib::{prelude::*, translate::*};
 #[cfg(all(not(unix), docsrs))]
-use socket::{AsRawFd, FromRawFd, IntoRawFd, RawFd};
+use socket::{AsFd, AsRawFd, BorrowedFd, FromRawFd, IntoRawFd, RawFd};
 
 glib::wrapper! {
     #[doc(alias = "GFileDescriptorBased")]
@@ -27,14 +27,13 @@ impl AsRawFd for FileDescriptorBased {
     }
 }
 
-mod sealed {
-    pub trait Sealed {}
-    impl<T: super::IsA<super::FileDescriptorBased>> Sealed for T {}
+impl AsFd for FileDescriptorBased {
+    fn as_fd(&self) -> BorrowedFd<'_> {
+        unsafe { BorrowedFd::borrow_raw(self.as_raw_fd()) }
+    }
 }
 
-pub trait FileDescriptorBasedExtManual:
-    sealed::Sealed + IsA<FileDescriptorBased> + 'static
-{
+pub trait FileDescriptorBasedExtManual: IsA<FileDescriptorBased> + 'static {
     #[doc(alias = "g_file_descriptor_based_get_fd")]
     #[doc(alias = "get_fd")]
     fn fd<T: FromRawFd>(&self) -> T {
