@@ -1,10 +1,9 @@
 // Take a look at the license at the top of the repository in the LICENSE file.
 
+mod async_test;
 mod boxed_derive;
 mod clone;
-mod clone_old;
 mod closure;
-mod closure_old;
 mod derived_properties_attribute;
 mod downgrade_derive;
 mod enum_derive;
@@ -19,7 +18,7 @@ mod variant_derive;
 mod utils;
 
 use flags_attribute::AttrInput;
-use proc_macro::{TokenStream, TokenTree};
+use proc_macro::TokenStream;
 use proc_macro2::Span;
 use syn::{parse_macro_input, DeriveInput};
 use utils::{parse_nested_meta_items_from_stream, NestedMetaItem};
@@ -335,18 +334,7 @@ use utils::{parse_nested_meta_items_from_stream, NestedMetaItem};
 /// ```
 #[proc_macro]
 pub fn clone(item: TokenStream) -> TokenStream {
-    // Check if this is an old-style clone macro invocation.
-    // These always start with an '@' punctuation.
-    let Some(first) = item.clone().into_iter().next() else {
-        return syn::Error::new(Span::call_site(), "expected a closure or async block")
-            .to_compile_error()
-            .into();
-    };
-
-    match first {
-        TokenTree::Punct(ref p) if p.to_string() == "@" => clone_old::clone_inner(item),
-        _ => clone::clone_inner(item),
-    }
+    clone::clone_inner(item)
 }
 
 /// Macro for creating a [`Closure`] object. This is a wrapper around [`Closure::new`] that
@@ -362,7 +350,7 @@ pub fn clone(item: TokenStream) -> TokenStream {
 ///
 /// Similarly to [`clone!`](crate::clone!), this macro can be useful in combination with signal
 /// handlers to reduce boilerplate when passing references. Unique to `Closure` objects is the
-/// ability to watch an object using a the `#[watch]` attribute. Only an [`Object`] value can be
+/// ability to watch an object using the `#[watch]` attribute. Only an [`Object`] value can be
 /// passed to `#[watch]`, and only one object can be watched per closure. When an object is watched,
 /// a weak reference to the object is held in the closure. When the object is destroyed, the
 /// closure will become invalidated: all signal handlers connected to the closure will become
@@ -504,18 +492,7 @@ pub fn clone(item: TokenStream) -> TokenStream {
 /// ```
 #[proc_macro]
 pub fn closure(item: TokenStream) -> TokenStream {
-    // Check if this is an old-style closure macro invocation.
-    // These always start with an '@' punctuation.
-    let Some(first) = item.clone().into_iter().next() else {
-        return syn::Error::new(Span::call_site(), "expected a closure")
-            .to_compile_error()
-            .into();
-    };
-
-    match first {
-        TokenTree::Punct(ref p) if p.to_string() == "@" => closure_old::closure_inner(item, "new"),
-        _ => closure::closure_inner(item, "new"),
-    }
+    closure::closure_inner(item, "new")
 }
 
 /// The same as [`closure!`](crate::closure!) but uses [`Closure::new_local`] as a constructor.
@@ -525,24 +502,11 @@ pub fn closure(item: TokenStream) -> TokenStream {
 /// [`Closure::new_local`]: ../glib/closure/struct.Closure.html#method.new_local
 #[proc_macro]
 pub fn closure_local(item: TokenStream) -> TokenStream {
-    // Check if this is an old-style closure macro invocation.
-    // These always start with an '@' punctuation.
-    let Some(first) = item.clone().into_iter().next() else {
-        return syn::Error::new(Span::call_site(), "expected a closure")
-            .to_compile_error()
-            .into();
-    };
-
-    match first {
-        TokenTree::Punct(ref p) if p.to_string() == "@" => {
-            closure_old::closure_inner(item, "new_local")
-        }
-        _ => closure::closure_inner(item, "new_local"),
-    }
+    closure::closure_inner(item, "new_local")
 }
 
-/// Derive macro for register a Rust enum in the GLib type system and derive the
-/// the [`glib::Value`] traits.
+/// Derive macro to register a Rust enum in the GLib type system and derive the
+/// [`glib::Value`] traits.
 ///
 /// # Example
 ///
@@ -561,8 +525,16 @@ pub fn closure_local(item: TokenStream) -> TokenStream {
 /// }
 /// ```
 ///
+/// When using the [`Properties`] macro with enums that derive [`Enum`], the
+/// default value can be explicitly set via the `builder` parameter of the
+/// `#[property]` attribute. If the enum implements or derives
+/// `Default`, you can specify that should be the default value
+/// via the `default` parameter. See [here](Properties#supported-types) for
+/// details.
+///
 /// An enum can be registered as a dynamic type by setting the derive macro
 /// helper attribute `enum_dynamic`:
+///
 /// ```ignore
 /// use glib::prelude::*;
 /// use glib::subclass::prelude::*;
@@ -576,7 +548,7 @@ pub fn closure_local(item: TokenStream) -> TokenStream {
 /// ```
 ///
 /// As a dynamic type, an enum must be explicitly registered when the system
-/// loads the implementation (see [`TypePlugin`] and [`TypeModule`].
+/// loads the implementation (see [`TypePlugin`] and [`TypeModule`]).
 /// Therefore, whereas an enum can be registered only once as a static type,
 /// it can be registered several times as a dynamic type.
 ///
@@ -590,9 +562,10 @@ pub fn closure_local(item: TokenStream) -> TokenStream {
 /// when registering an enum as a dynamic type:
 ///
 /// - lazy registration: by default an enum is registered as a dynamic type
-/// when the system loads the implementation (e.g. when the module is loaded).
-/// Optionally setting `lazy_registration` to `true` postpones registration on
-/// the first use (when `static_type()` is called for the first time):
+///   when the system loads the implementation (e.g. when the module is loaded).
+///   Optionally setting `lazy_registration` to `true` postpones registration on
+///   the first use (when `static_type()` is called for the first time):
+///
 /// ```ignore
 /// #[derive(Debug, Copy, Clone, PartialEq, Eq, glib::Enum)]
 /// #[enum_type(name = "MyEnum")]
@@ -603,8 +576,9 @@ pub fn closure_local(item: TokenStream) -> TokenStream {
 /// ```
 ///
 /// - registration within [`TypeModule`] subclass or within [`TypePlugin`]
-/// subclass: an enum is usually registered as a dynamic type within a
-/// [`TypeModule`] subclass:
+///   subclass: an enum is usually registered as a dynamic type within a
+///   [`TypeModule`] subclass:
+///
 /// ```ignore
 /// #[derive(Debug, Copy, Clone, PartialEq, Eq, glib::Enum)]
 /// #[enum_type(name = "MyModuleEnum")]
@@ -629,6 +603,7 @@ pub fn closure_local(item: TokenStream) -> TokenStream {
 ///
 /// Optionally setting `plugin_type` allows to register an enum as a dynamic
 /// type within a [`TypePlugin`] subclass that is not a [`TypeModule`]:
+///
 /// ```ignore
 /// #[derive(Debug, Copy, Clone, PartialEq, Eq, glib::Enum)]
 /// #[enum_type(name = "MyPluginEnum")]
@@ -671,7 +646,7 @@ pub fn enum_derive(input: TokenStream) -> TokenStream {
 /// Default name is the flag identifier in CamelCase and default nick
 /// is the identifier in kebab-case.
 /// Combined flags should not be registered with the `GType` system
-/// and so needs to be tagged with the `#[flags_value(skip)]` attribute.
+/// and so need to be tagged with the `#[flags_value(skip)]` attribute.
 ///
 /// # Example
 ///
@@ -705,7 +680,7 @@ pub fn enum_derive(input: TokenStream) -> TokenStream {
 /// ```
 ///
 /// As a dynamic type, the flags must be explicitly registered when the system
-/// loads the implementation (see [`TypePlugin`] and [`TypeModule`].
+/// loads the implementation (see [`TypePlugin`] and [`TypeModule`]).
 /// Therefore, whereas the flags can be registered only once as a static type,
 /// they can be registered several times as a dynamic type.
 ///
@@ -719,9 +694,10 @@ pub fn enum_derive(input: TokenStream) -> TokenStream {
 /// registering the flags as a dynamic type:
 ///
 /// - lazy registration: by default the flags are registered as a dynamic type
-/// when the system loads the implementation (e.g. when the module is loaded).
-/// Optionally setting `lazy_registration` to `true` postpones registration on
-/// the first use (when `static_type()` is called for the first time):
+///   when the system loads the implementation (e.g. when the module is loaded).
+///   Optionally setting `lazy_registration` to `true` postpones registration on
+///   the first use (when `static_type()` is called for the first time):
+///
 /// ```ignore
 /// #[glib::flags(name = "MyFlags")]
 /// #[flags_dynamic(lazy_registration = true)]
@@ -731,8 +707,9 @@ pub fn enum_derive(input: TokenStream) -> TokenStream {
 /// ```
 ///
 /// - registration within [`TypeModule`] subclass or within [`TypePlugin`]
-/// subclass: the flags are usually registered as a dynamic type within a
-/// [`TypeModule`] subclass:
+///   subclass: the flags are usually registered as a dynamic type within a
+///   [`TypeModule`] subclass:
+///
 /// ```ignore
 /// #[glib::flags(name = "MyModuleFlags")]
 /// #[flags_dynamic]
@@ -920,7 +897,7 @@ pub fn shared_boxed_derive(input: TokenStream) -> TokenStream {
 /// type Interfaces = ();
 /// ```
 ///
-/// If no `new()` or `with_class()` method is provide, the macro adds a `new()`
+/// If no `new()` or `with_class()` method is provided, the macro adds a `new()`
 /// implementation calling `Default::default()`. So the type needs to implement
 /// `Default`, or this should be overridden.
 ///
@@ -932,6 +909,7 @@ pub fn shared_boxed_derive(input: TokenStream) -> TokenStream {
 ///
 /// An object subclass can be registered as a dynamic type by setting the macro
 /// helper attribute `object_class_dynamic`:
+///
 /// ```ignore
 /// #[derive(Default)]
 /// pub struct MyType;
@@ -942,7 +920,7 @@ pub fn shared_boxed_derive(input: TokenStream) -> TokenStream {
 /// ```
 ///
 /// As a dynamic type, an object subclass must be explicitly registered when
-/// the system loads the implementation (see [`TypePlugin`] and [`TypeModule`].
+/// the system loads the implementation (see [`TypePlugin`] and [`TypeModule`]).
 /// Therefore, whereas an object subclass can be registered only once as a
 /// static type, it can be registered several times as a dynamic type.
 ///
@@ -956,10 +934,11 @@ pub fn shared_boxed_derive(input: TokenStream) -> TokenStream {
 /// when registering an object subclass as a dynamic type:
 ///
 /// - lazy registration: by default an object subclass is registered as a
-/// dynamic type when the system loads the implementation (e.g. when the module
-/// is loaded). Optionally setting `lazy_registration` to `true` postpones
-/// registration on the first use (when `static_type()` is called for the first
-/// time):
+///   dynamic type when the system loads the implementation (e.g. when the module
+///   is loaded). Optionally setting `lazy_registration` to `true` postpones
+///   registration on the first use (when `static_type()` is called for the first
+///   time):
+///
 /// ```ignore
 /// #[derive(Default)]
 /// pub struct MyType;
@@ -970,8 +949,9 @@ pub fn shared_boxed_derive(input: TokenStream) -> TokenStream {
 /// ```
 ///
 /// - registration within [`TypeModule`] subclass or within [`TypePlugin`]
-/// subclass: an object subclass is usually registered as a dynamic type within
-/// a [`TypeModule`] subclass:
+///   subclass: an object subclass is usually registered as a dynamic type within
+///   a [`TypeModule`] subclass:
+///
 /// ```ignore
 /// #[derive(Default)]
 /// pub struct MyModuleType;
@@ -996,6 +976,7 @@ pub fn shared_boxed_derive(input: TokenStream) -> TokenStream {
 ///
 /// Optionally setting `plugin_type` allows to register an object subclass as a
 /// dynamic type within a [`TypePlugin`] subclass that is not a [`TypeModule`]:
+///
 /// ```ignore
 /// #[derive(Default)]
 /// pub struct MyPluginType;
@@ -1032,10 +1013,10 @@ pub fn object_subclass(_attr: TokenStream, item: TokenStream) -> TokenStream {
 /// This adds implementations for the `get_type()` method, which should probably never be defined
 /// differently.
 ///
-/// It provides default values for the `Prerequisites` type parameter. If this present, the macro
+/// It provides default values for the `Prerequisites` type parameter. If this is present, the macro
 /// will use the provided value instead of the default.
 ///
-/// `Prerequisites` is interfaces for types that require a specific base class or interfaces.
+/// `Prerequisites` are interfaces for types that require a specific base class or interfaces.
 ///
 /// ```ignore
 /// type Prerequisites = ();
@@ -1053,7 +1034,7 @@ pub fn object_subclass(_attr: TokenStream, item: TokenStream) -> TokenStream {
 /// ```
 ///
 /// As a dynamic type, an object interface must be explicitly registered when
-/// the system loads the implementation (see [`TypePlugin`] and [`TypeModule`].
+/// the system loads the implementation (see [`TypePlugin`] and [`TypeModule`]).
 /// Therefore, whereas an object interface can be registered only once as a
 /// static type, it can be registered several times as a dynamic type.
 ///
@@ -1067,9 +1048,10 @@ pub fn object_subclass(_attr: TokenStream, item: TokenStream) -> TokenStream {
 /// behaviors when registering an object interface as a dynamic type:
 ///
 /// - lazy registration: by default an object interface is registered as a
-/// dynamic type when the system loads the implementation (e.g. when the module
-/// is loaded). Optionally setting `lazy_registration` to `true` postpones
-/// registration on the first use (when `type_()` is called for the first time):
+///   dynamic type when the system loads the implementation (e.g. when the module
+///   is loaded). Optionally setting `lazy_registration` to `true` postpones
+///   registration on the first use (when `type_()` is called for the first time):
+///
 /// ```ignore
 /// pub struct MyInterface {
 ///     parent: glib::gobject_ffi::GTypeInterface,
@@ -1080,8 +1062,9 @@ pub fn object_subclass(_attr: TokenStream, item: TokenStream) -> TokenStream {
 /// ```
 ///
 /// - registration within [`TypeModule`] subclass or within [`TypePlugin`]
-/// subclass: an object interface is usually registered as a dynamic type
-/// within a [`TypeModule`] subclass:
+///   subclass: an object interface is usually registered as a dynamic type
+///   within a [`TypeModule`] subclass:
+///
 /// ```ignore
 /// pub struct MyModuleInterface {
 ///     parent: glib::gobject_ffi::GTypeInterface,
@@ -1106,6 +1089,7 @@ pub fn object_subclass(_attr: TokenStream, item: TokenStream) -> TokenStream {
 ///
 /// Optionally setting `plugin_type` allows to register an object interface as
 /// a dynamic type within a [`TypePlugin`] subclass that is not a [`TypeModule`]:
+///
 /// ```ignore
 /// pub struct MyPluginInterface {
 ///     parent: glib::gobject_ffi::GTypeInterface,
@@ -1249,16 +1233,16 @@ pub fn downgrade(input: TokenStream) -> TokenStream {
 /// enum variant, or just `s` if this is a C-style enum. Some additional attributes are supported
 /// for enums:
 /// - `#[variant_enum(repr)]` to serialize the enum variant as an integer type instead of `s`.  The
-/// `#[repr]` attribute must also be specified on the enum with a sized integer type, and the type
-/// must implement `Copy`.
+///   `#[repr]` attribute must also be specified on the enum with a sized integer type, and the type
+///   must implement `Copy`.
 /// - `#[variant_enum(enum)]` uses [`EnumClass`] to serialize/deserialize as nicks. Meant for use
-/// with [`glib::Enum`](Enum).
+///   with [`glib::Enum`](Enum).
 /// - `#[variant_enum(flags)]` uses [`FlagsClass`] to serialize/deserialize as nicks. Meant for use
-/// with [`glib::flags`](macro@flags).
+///   with [`glib::flags`](macro@flags).
 /// - `#[variant_enum(enum, repr)]` serializes as `i32`. Meant for use with [`glib::Enum`](Enum).
-/// The type must also implement `Copy`.
+///   The type must also implement `Copy`.
 /// - `#[variant_enum(flags, repr)]` serializes as `u32`. Meant for use with
-/// [`glib::flags`](macro@flags).
+///   [`glib::flags`](macro@flags).
 ///
 /// # Example
 ///
@@ -1352,7 +1336,8 @@ pub fn cstr_bytes(item: TokenStream) -> TokenStream {
 /// | `construct` | Specify that the property is construct property. Ensures that the property is always set during construction (if not explicitly then the default value is used). The use of a custom internal setter is supported. | | `#[property(get, construct)]` or `#[property(get, set = set_prop, construct)]` |
 /// | `construct_only` | Specify that the property is construct only. This will not generate a public setter and only allow the property to be set during object construction. The use of a custom internal setter is supported. | | `#[property(get, construct_only)]` or `#[property(get, set = set_prop, construct_only)]` |
 /// | `builder(<required-params>)[.ident]*` | Used to input required params or add optional Param Spec builder fields | | `#[property(builder(SomeEnum::default()))]`, `#[builder().default_value(1).minimum(0).maximum(5)]`, etc.  |
-/// | `default` | Sets the `default_value` field of the Param Spec builder | | `#[property(default = 1)]` |
+/// | `default` | Sets the param spec builder field to the default value | | `#[property(default)]` |
+/// | `default = expr` | Sets the `default_value` field of the Param Spec builder | | `#[property(default = 1)]` |
 /// | `<optional-pspec-builder-fields> = expr` | Used to add optional Param Spec builder fields | | `#[property(minimum = 0)` , `#[property(minimum = 0, maximum = 1)]`, etc. |
 /// | `<optional-pspec-builder-fields>` | Used to add optional Param Spec builder fields | | `#[property(explicit_notify)]` , `#[property(construct_only)]`, etc. |
 ///
@@ -1367,6 +1352,10 @@ pub fn cstr_bytes(item: TokenStream) -> TokenStream {
 /// * `connect_$property_notify()`
 /// * `notify_$property()`
 ///
+/// # Documentation
+///
+/// Doc comments preceding a `#[property]` attribute will be copied to the generated getter and setter methods. You can specify different comments by the getter and setter by using `# Getter` and `# Setter` headings. The text under the header will be copied to the respective method.
+///
 /// ## Extension trait
 /// You can choose to move the method definitions to a trait by using `#[properties(wrapper_type = super::MyType, ext_trait = MyTypePropertiesExt)]`.
 /// The trait name is optional, and defaults to `MyTypePropertiesExt`, where `MyType` is extracted from the wrapper type.
@@ -1385,6 +1374,9 @@ pub fn cstr_bytes(item: TokenStream) -> TokenStream {
 /// Optional types also require the `nullable` attribute: without it, the generated setter on the wrapper type
 /// will take `T` instead of `Option<T>`, preventing the user from ever calling the setter with a `None` value.
 ///
+/// Notice: For enums that derive [`Enum`] or are C-style enums, you must explicitly specify the
+/// default value of the enum using the `builder` parameter in the `#[property]` attribute.
+///
 /// ## Adding support for custom types
 /// ### Types wrapping an existing <code>T: [ToValue] + [HasParamSpec]</code>
 /// If you have declared a newtype as
@@ -1409,7 +1401,7 @@ pub fn cstr_bytes(item: TokenStream) -> TokenStream {
 ///
 /// # Example
 /// ```
-/// use std::cell::RefCell;
+/// use std::cell::{Cell, RefCell};
 /// use glib::prelude::*;
 /// use glib::subclass::prelude::*;
 /// use glib_macros::Properties;
@@ -1420,6 +1412,14 @@ pub fn cstr_bytes(item: TokenStream) -> TokenStream {
 ///     nick: String,
 /// }
 ///
+/// #[derive(Debug, Copy, Clone, PartialEq, Eq, glib::Enum, Default)]
+/// #[enum_type(name = "MyEnum")]
+/// enum MyEnum {
+///     #[default]
+///     Val,
+///     OtherVal
+/// }
+///
 /// pub mod imp {
 ///     use std::rc::Rc;
 ///
@@ -1430,7 +1430,9 @@ pub fn cstr_bytes(item: TokenStream) -> TokenStream {
 ///     pub struct Foo {
 ///         #[property(get, set = Self::set_fizz)]
 ///         fizz: RefCell<String>,
+///         /// The author's name
 ///         #[property(name = "author-name", get, set, type = String, member = name)]
+///         /// The author's childhood nickname
 ///         #[property(name = "author-nick", get, set, type = String, member = nick)]
 ///         author: RefCell<Author>,
 ///         #[property(get, set, explicit_notify, lax_validation)]
@@ -1443,6 +1445,19 @@ pub fn cstr_bytes(item: TokenStream) -> TokenStream {
 ///         optional: RefCell<Option<String>>,
 ///         #[property(get, set)]
 ///         smart_pointer: Rc<RefCell<String>>,
+///         #[property(get, set, builder(MyEnum::Val))]
+///         my_enum: Cell<MyEnum>,
+///         #[property(get, set, default)]
+///         my_enum_with_default: Cell<MyEnum>,
+///         /// # Getter
+///         ///
+///         /// Get the value of the property `extra_comments`
+///         ///
+///         /// # Setter
+///         ///
+///         /// This is the comment for the setter of the `extra_comments` field.
+///         #[property(get, set)]
+///         extra_comments: RefCell<bool>,
 ///     }
 ///     
 ///     #[glib::derived_properties]
@@ -1591,3 +1606,22 @@ pub fn derive_value_delegate(input: TokenStream) -> TokenStream {
     let input = parse_macro_input!(input as value_delegate_derive::ValueDelegateInput);
     value_delegate_derive::impl_value_delegate(input).unwrap()
 }
+
+/// An attribute macro for writing asynchronous test functions.
+///
+/// This macro is designed to wrap an asynchronous test function and ensure that
+/// it runs within a `glib::MainContext`. It helps in writing async tests that
+/// require the use of an event loop for the asynchronous execution.
+///
+/// # Example
+///
+/// ```
+/// #[glib::async_test]
+/// async fn my_async_test() {
+///     // Test code that runs asynchronously
+/// }
+/// ```
+#[proc_macro_attribute]
+pub fn async_test(args: TokenStream, item: TokenStream) -> TokenStream {
+    async_test::async_test(args, item)
+}
