@@ -9,9 +9,12 @@
 #[cfg(all(test, feature = "_benchmarks"))]
 extern crate test;
 
-pub use self::decoder::{DecodingError, LoopCount, WebPDecoder};
+pub use self::decoder::{
+    DecodingError, LoopCount, UpsamplingMethod, WebPDecodeOptions, WebPDecoder,
+};
 pub use self::encoder::{ColorType, EncoderParams, EncodingError, WebPEncoder};
 
+mod alpha_blending;
 mod decoder;
 mod encoder;
 mod extended;
@@ -20,5 +23,7 @@ mod loop_filter;
 mod lossless;
 mod lossless_transform;
 mod transform;
+mod vp8_arithmetic_decoder;
+mod yuv;
 
 pub mod vp8;
