@@ -1,6 +1,3 @@
-// Only run test when parser is enabled
-#![cfg(any(feature = "pom_parser", feature = "nom_parser"))]
-
 use lopdf::Result;
 use tempfile::tempdir;
 
